/*
  Copyright 2003 Rikard Bjrklind, Mikael Gransell

  This file is part of dc-qt.

  dc-qt is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  dc-qt is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with dc-qt; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mainwdgt.h"
#include <qpushbutton.h>
#include <qmenubar.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qpopupmenu.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qtoolbutton.h>
#include <qsplitter.h>
#include <qstatusbar.h>
#include <qdialog.h>
#include <qlayout.h>
#include <qaction.h>
#include <qprogressbar.h>

#include <qurloperator.h>
#include <qnetworkprotocol.h>
#include <qnetwork.h>
#include <qprocess.h>
#include <unistd.h>
#include <qprogressdialog.h>

#include "dc_settings.h"
#include "preferences_dlg.h"
#include "searchwdgt.h"
#include "pixmaps.h"
#include "config.h"
#include "helpwindow.h"
#include "dc_hub.h"
#include "dctc_hub.h"
#include "transfer_widget.h"
#include "hubwdgt.h"

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

// Some globals
DebugDlg* debugWin;
dc_settings settings;
Pixmap *pixmaps;
MainWdgt* MainWdgt::instance;

MainWdgt::MainWdgt( QWidget* parent, const char* name ) : QMainWindow( parent, name )
{
	progressDlg = 0;
	pixmaps = new Pixmap;

	// Set the application icon
	setIcon( pixmaps->getIcon() );

	//     QVBox *mainbox = new QVBox(this, "mainvbox");
	QSplitter *mainbox = new QSplitter(Qt::Vertical,this,"mainsplitter");

	hubTabs = new QTabWidget( mainbox, "Hub Tabs" );
	QObject::connect(hubTabs, SIGNAL(currentChanged(QWidget*)), this, SLOT(selectHub(QWidget*)));

	// Create the global transferWindow
	//     transferWin = new TransferDlg(mainbox,"Transfer window");

	// estrato adding new transfer dialog here..
	transferWidget = new transfer_widget(mainbox, "Transfers Window");

	QPopupMenu* file = new QPopupMenu( this, "File menu" );
	QPopupMenu* help = new QPopupMenu( this, "Help menu" );

	// Set our popupmenus in the main window menubar
	menuBar()->insertItem( "&Actions", file );
	menuBar()->insertItem( "&Help", help );

	QIconSet::setIconSize(QIconSet::Large,QSize(22,22));
	QIconSet newIcon(pixmaps->getNew());
	QIconSet closeIcon(pixmaps->getClose());
	QIconSet connectIcon(pixmaps->getConnect());
	QIconSet disIcon(pixmaps->getDisconnect());
	QIconSet searchIcon(pixmaps->getSearch());
	QIconSet configureIcon(pixmaps->getConfigure());
	QIconSet exitIcon(pixmaps->getExit());
	QIconSet reshareIcon(pixmaps->getReshare());

	aNewHub = new QAction("New Hub", newIcon, "&New hub", CTRL+Key_N, this, "newhubtab");
	aNewHub->setToolTip("Creates a new hub tab");
	aCloseHub = new QAction("Close Hub", closeIcon, "C&lose hub", CTRL+Key_X, this, "closehubtab");
	aCloseHub->setToolTip("Deletes the selected hub tab");
	aConnect = new QAction("Connect", connectIcon, "&Connect", CTRL+Key_C, this, "connect");
	aConnect->setToolTip("Opens the connection dialog");
	aDisconnect = new QAction("Disconnect", disIcon, "&Disconnect", CTRL+Key_D, this, "disconnect");
	aDisconnect->setToolTip("Disconnects the active hub tab");
	aReshare = new QAction("Reshare",reshareIcon,"&Reshare",CTRL+Key_R,this,"reshare");
	aReshare->setToolTip("Rebuilds the shared file database");
	aSearch = new QAction("Search", searchIcon, "&Search", CTRL+Key_S, this, "search");
	aSearch->setToolTip("Opens the search dialog. There is one search dialog per hub.");
	aConfig = new QAction("Preferences", configureIcon, "&Preferences", CTRL+Key_P, this, "prefs");
	aConfig->setToolTip("Opens the preferences dialog.");
	aExit = new QAction("Quit", exitIcon, "&Quit", CTRL+Key_Q,this, "quit");
	aExit->setToolTip("Quits the program");
	QObject::connect(aNewHub, SIGNAL(activated()), this, SLOT(newHubTab()));
	QObject::connect(aCloseHub, SIGNAL(activated()), this, SLOT(closeHubTab()));
	QObject::connect(aConnect, SIGNAL(activated()), this, SLOT(connectHub()));
	QObject::connect(aDisconnect, SIGNAL(activated()), this, SLOT(disHub()));
	QObject::connect(aReshare,SIGNAL(activated()),this,SLOT(reshare()));
	QObject::connect(aSearch, SIGNAL(activated()), this, SLOT(searchHub()));
	QObject::connect(aConfig, SIGNAL(activated()), this, SLOT(showPreferences()));
	QObject::connect(aExit, SIGNAL(activated()), this, SLOT(close()));
	aNewHub->addTo(file);
	aCloseHub->addTo(file);
	aConnect->addTo(file);
	aDisconnect->addTo(file);
	aDisconnect->setEnabled(false);
	aReshare->addTo(file);
	aSearch->addTo(file);
	aConfig->addTo(file);
	aExit->addTo(file);

	QAction *aHelpContents = new QAction("Contents","&Contents",Key_F1,this,"helpcontentsaction");
	QObject::connect(aHelpContents,SIGNAL(activated()),this,SLOT(displayHelp()));
	aHelpContents->addTo(help);
	help->insertItem( "Newest &version",this,SLOT(checkVersion()));
	help->insertItem( "&About", this, SLOT(displayAbout()) );
	help->insertItem( "About &Qt",this, SLOT(displayAboutQt()) );

	newHubTab();

	// Add all things to our main window
	setCaption( "dc-qt " VERSION );
	setCentralWidget( mainbox );

	// Create the toolbar
	QToolBar * tools = new QToolBar( this, "toolbar" );
	tools->setHorizontallyStretchable(true);
	tools->setLabel( "Tools" );

	setUsesBigPixmaps(true);
	setUsesTextLabel(true);

	aNewHub->addTo(tools);
	aCloseHub->addTo(tools);
	aSearch->addTo(tools);
	aConnect->addTo(tools);
	aDisconnect->addTo(tools);
	//     aConfig->addTo(tools);
	aExit->addTo(tools);

	//     tools->boxLayout()->insertStretch(1);
	// Create the global debugWindow

	debugWin = new DebugDlg();
#ifdef _DEBUG
	debugWin->show();
#endif

	switch(settings.init())
	{
	case dc_settings::SUCCESS:
		break;
	case dc_settings::NO_SETTINGS:
		showPreferences();
		break;
	case dc_settings::ERROR_LOADING_PREFS:
		QMessageBox::critical(this, "Error", "Error loading preferences file.", "Ok");
		break;
		//     case dc_settings::ERROR_LOADING_FAVOURITES:
		//	  QMessageBox::critical(this, "Error", "Error loading the favourites file.","Ok");
		//	  break;
	default:
		// This should not happen
		break;
	}
	QFont font;
	font.fromString( settings.get_setting(dc_settings::FONT) );
	QApplication::setFont(font,true);

	statusBar()->message("Ready.",2000);
	statusBar()->setSizeGripEnabled(true);

	instance = this;
}

void MainWdgt::setHubName(const QString &name, QWidget *wdgt)
{
	hubTabs->changeTab(wdgt,name);
}

void MainWdgt::showPreferences()
{
	preferences_dlg *pdlg = new preferences_dlg(this,"preferencesdialog",true);
	pdlg->exec();
	delete pdlg;
}

void MainWdgt::displayAbout()
{
	QMessageBox::about(this,"About dcqt","This is dc-qt " VERSION "\ndc-qt is a front end for the direct connect client dctc.\nAuthors:\n\nRikard Bjrklind (olof@linux.nu)\nMikael Gransell (migr8915@student.uu.se) \nArsenij Vodjanov (aikiokio@dangerous-minds.com)\n\nhttp://dc-qt.sourceforge.net");
}

void MainWdgt::displayAboutQt()
{
	QMessageBox::aboutQt(this);
}

void MainWdgt::newHubTab()
{
	HubWdgt* hub = new HubWdgt( transferWidget, hubTabs, "hub" );
	// Add our new hub
	hubTabs->addTab( hub, "New Hub" );
	// Set the id of our tab so we can remove it later
	hub->setId( hubTabs->indexOf(hub) );
	QObject::connect( hub, SIGNAL(gotName(const QString&, QWidget*)), this, SLOT(setHubName(const QString&, QWidget*)));
	QObject::connect( hub, SIGNAL(multiSearchResult(const QString&,const QString&,const QString&,const QString&,const QString&)),
	                  this, SLOT(dispatchSearchResult(const QString&,const QString&,const QString&,const QString&,const QString&)));
	QObject::connect ( hub, SIGNAL(disconnected(int)), this, SLOT(hubDisconnected(int)));
	QObject::connect( hub,SIGNAL(showStatus(const QString&)),this,SLOT(showStatus(const QString&)));
	QObject::connect(hub,SIGNAL(showProgress(const QString&,const QString&)),
	                 this,SLOT(showProgress(const QString&,const QString&)));
	QObject::connect(hub,SIGNAL(connected(int)),this,SLOT(hubConnected(int )));
	hubTabs->showPage(hub);
}

void MainWdgt::hubDisconnected(int id)
{
	if(hubTabs->currentPageIndex()==id)
	{
		aConnect->setEnabled(true);
		aDisconnect->setEnabled(false);
	}
	statusBar()->message("Hub " + hubTabs->label(id) + " disconnected",3000);
}

void MainWdgt::hubConnected(int id)
{
	if(hubTabs->currentPageIndex()==id)
	{
		aConnect->setEnabled(false);
		aDisconnect->setEnabled(true);
	}
}

void MainWdgt::dispatchSearchResult(const QString& u,const QString& f,const QString& s,const QString& sl,const QString& hubname)
{
	// Find the right hub
	for(int i=0;i < hubTabs->count();i++)
		if( hubTabs->label(i) == hubname ) // found it
		{
			HubWdgt *w = (HubWdgt *)hubTabs->page(i);
			w->handleSearchResult(u,f,s,sl,hubname);
		}
}

void MainWdgt::closeHubTab(  )
{
	HubWdgt* hub = (HubWdgt*)hubTabs->currentPage();
	if(!hub) return;
	if(hub->isConnected())
		switch( QMessageBox::warning( this, "dc-qt",
		                              "Closing the hub tab will disconnect you from the hub and abort all transfers.\n",
		                              "Ok",
		                              "Cancel", 0, 0, 1 ) )
		{
		case 0: // The user clicked the Ok again button or pressed Enter
			hubTabs->removePage( hub );
			hub->disconnect();
			delete hub;
			break;
		case 1: // The user clicked the Cancel button or pressed Escape
			break;

		}
	else
	{
		hubTabs->removePage( hub );
		hub->disconnect();
		delete hub;
	}
}

/// Connects the current hub
void MainWdgt::connectHub()
{
	HubWdgt* hub = (HubWdgt*)hubTabs->currentPage();
	if(!hub) return;
	if(hub->connect())
	{
		aConnect->setEnabled(false);
		aDisconnect->setEnabled(true);
	}
}

/// Disconnects the current hub
void MainWdgt::disHub()
{
	HubWdgt* hub = (HubWdgt*)hubTabs->currentPage();
	if(!hub) return;
	hub->disconnect();
	aConnect->setEnabled(true);
	aDisconnect->setEnabled(false);
}

/// Brings up the search dialog for the active hub
void MainWdgt::searchHub()
{
	HubWdgt* hub = (HubWdgt*)hubTabs->currentPage();
	if(!hub) return;
	hub->openSearchDlg();
}

void MainWdgt::closeEvent( QCloseEvent *e)
{
	bool ask = false;
	int numclients;
	// Check if there are any connected hubs
	for(int i=0; i < hubTabs->count();i++)
	{
		HubWdgt *hub = (HubWdgt*)hubTabs->page(i);
		if( hub->isConnected() ) ask = true;
	}

	if( ask )
	{
		switch( QMessageBox::warning( this, "dc-qt",
		                              "Do you want to disconnect from all hubs before exiting?.\n",
		                              "Quit && &disconnect",
		                              "Just &Quit",
		                              "&Cancel", 0, 2 ) )
		{
		case 0:
			for(int i=0; i < hubTabs->count();i++)
			{
				HubWdgt *hub = (HubWdgt*)hubTabs->page(i);
				hub->disconnect();
			}
			usleep(500000);
			// We have closed all connections, see if we can kill the master as well.
			numclients = dc_hub::getNumClients();
#ifdef _DEBUG
			cout << "num clients running=" << numclients << "\n";
#endif
			if(  numclients == 0)
			{
				dctc_hub::killMaster();
			}
			e->accept();
			break;
		case 1:
			e->accept();
			break;
		case 2:
			e->ignore();
		}
	}
	else
	{
		// We didn't need to ask since all hubs where disconnected.
		// There could still be an unused master though. So kill it.
		numclients = dc_hub::getNumClients();
		if(  numclients == 0 ) dctc_hub::killMaster();
		e->accept();
	}
}

void MainWdgt::selectHub(QWidget *w)
{
	HubWdgt *h = (HubWdgt *)w;
	if(h->isConnected())
	{
		aConnect->setEnabled(false);
		aDisconnect->setEnabled(true);
	}
	else
	{
		aConnect->setEnabled(true);
		aDisconnect->setEnabled(false);
	}

}

void MainWdgt::displayHelp()
{
	HelpWindow *hw = new HelpWindow("index.html","/usr/local/doc/dc-qt",0,"scheisse");
	hw->show();
}

/// Checks the newest version by obtaining a file from the dc-qt web page on sourceforge.
void MainWdgt::checkVersion()
{
	checkDCTCVersion();
	qInitNetworkProtocols();
	QUrlOperator *op = new QUrlOperator( "http://dc-qt.sourceforge.net" );
	op->get("version.txt");
	QObject::connect(op,SIGNAL(data(const QByteArray&, QNetworkOperation*)),
	                 this,SLOT(incomingData(const QByteArray&, QNetworkOperation*)));
	QObject::connect(op,SIGNAL(finished(QNetworkOperation*)),this,SLOT(versionCheckFinish(QNetworkOperation*)));
}

void MainWdgt::incomingData(const QByteArray &data, QNetworkOperation *op)
{
	if(versionData.isNull()) versionData = data;
	else
	{
		uint oldSize = versionData.size();
		versionData.resize(oldSize+data.size());
		for(uint i=0;i < data.size();i++)
			versionData[i+oldSize] = data[i];
	}
}

void MainWdgt::versionCheckFinish(QNetworkOperation *op)
{
	if(op->errorCode() == QNetworkProtocol::NoError)
	{
		QString text(versionData);
		QString version = text.section('\n',0,0);
		QString dctcversion = text.section('\n',1,1);
		QString changelog = text.section('\n',2);

		while(dctcVersion==QString::null) ;

		int curDctcV = 100*100*dctcVersion.section(".",0,0).toInt() +
		               100*dctcVersion.section(".",1,1).toInt() +
		               dctcVersion.section(".",2,2).toInt();
		int reqDctcV = 100*100*dctcversion.section(".",0,0).toInt() +
		               100*dctcversion.section(".",1,1).toInt() +
		               dctcversion.section(".",2,2).toInt();

		if(version==VERSION && curDctcV >= reqDctcV)
			QMessageBox::information(this,"Version checker",
			                         "Congratualtions, you have the latest version of dc-qt and the required version of DCTC.",QMessageBox::Yes);
		else
			QMessageBox::information(this,"Version checker",
			                         "Latest dc-qt version is " + version +
			                         "\nIt requires DCTC "+dctcversion+
			                         "\nYour version of DCTC is "+dctcVersion+
			                         "\nChangelog:\n"+changelog,QMessageBox::Ok);
	}
	else
	{
		QMessageBox::warning(this,"Version checker","Could not retrieve version, check your internet connection.",
		                     "Dammit!");
	}

}

static QProcess* dctcProc;

void MainWdgt::checkDCTCVersion()
{
	dctcProc = new QProcess((QString)"dctc",this,"dctcversionproc");
	connect(dctcProc,SIGNAL(processExited()),this,SLOT(dctcOutputReady()));
	dctcVersion = QString::null;
	dctcProc->start();
}

void MainWdgt::dctcOutputReady()
{
	dctcVersion = dctcProc->readLineStdout().section(" ",-1,-1).mid(1,6);
	delete dctcProc;
	dctcProc = 0;
}

void MainWdgt::showStatus(const QString& str)
{
	statusBar()->message(str,3000);
}

void MainWdgt::showProgress(const QString &msg,const QString &progress)
{
	int p = (int)progress.toFloat(0);
	QString dir = msg.section(':',1,1);
	if(p==0 && !progressDlg)
	{
		//progressBar = new QProgressBar(100,this,"progressbar");
		//progressBar->setProgress(0);
		//statusBar()->addWidget(progressBar,true);
		enableActions(false);
		progressDlg = new QProgressDialog("Building shared file database",QString::null,100,this);
		progressDlg->show();
		statusBar()->message("Building shared file database",1000);
	}
	else if(p==100)
	{
		progressDlg->setProgress(100);
		delete progressDlg;
		progressDlg = 0;
		//statusBar()->removeWidget(progressBar);
		//delete progressBar;
		//progressBar = 0;
		enableActions(true);
	}
	else
	{
		//progressBar->setProgress(p);
		progressDlg->setLabelText("Building shared file database\n" + dir);
		progressDlg->setProgress(p);
	}
}

void MainWdgt::enableActions(bool yes)
{
	aNewHub->setEnabled(yes);
	aCloseHub->setEnabled(yes);
	if(yes && !((HubWdgt*)hubTabs->currentPage())->isConnected()) aConnect->setEnabled(yes);
	aDisconnect->setEnabled(yes);
	aSearch->setEnabled(yes);
	aConfig->setEnabled(yes);
	aExit->setEnabled(yes);
}

void MainWdgt::reshare()
{
	// Reshare the currently selected hub, if connected.
	HubWdgt* hub = (HubWdgt*)hubTabs->currentPage();
	hub->reshare();
}

int MainWdgt::getNumConnections() const
{
	int ret = 0;
	for(int i=0; i < hubTabs->count();i++)
		if( ((HubWdgt*)hubTabs->page(i))->isConnected()) ret++;
	return ret;
}

const MainWdgt& MainWdgt::getInstance()
{
	return *instance;
}

/*
 *
 * $Log:
 *
 */
